@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import abc
-import functools
 import shutil
 import typing as t
 from pathlib import Path
@@ -17,11 +16,9 @@ except ImportError:
 
 
 T = t.TypeVar('T')
-StepFunc = t.Callable[['Context', Path, list[Path]], t.Any]
-UnboundStep = t.Union[StepFunc, 'Step'] | None
-PathCalc = t.Callable[['Context', Path, T], Path] | None
-BoundStep = t.Callable[[Path, list[Path]], t.Any]
+T2 = t.TypeVar('T2')
 BuildSettingsKey = t.Literal['input_dir', 'output_dir', 'working_dir', 'purge_dirs']
+ContextDir = t.Literal['input_dir', 'output_dir', 'working_dir']
 
 
 class InputBuildSettings(t.TypedDict, total=False):
@@ -71,31 +68,25 @@ class Context:
                  settings: BuildSettings,
                  rules: list[Rule]):
         self.settings = settings
-        self.rules = [(r, self.bind(r.step)) for r in rules]
+        self.rules: list[Rule] = []
+        for r in rules:
+            self.rules.append(r)
+            self.bind(r.step)
 
     @t.overload
-    def __getitem__(self, key: t.Literal['input_dir', 'output_dir', 'working_dir']) -> Path: ...
+    def __getitem__(self, key: ContextDir) -> Path: ...
     @t.overload
     def __getitem__(self, key: t.Literal['purge_dirs']) -> bool: ...
     def __getitem__(self, key):
         return self.settings[key]
 
-    def bind(self, step: UnboundStep) -> BoundStep | None:
+    def bind(self, step: Step | None):
         """
         Bind a Step to this Context. Calls the `bind()` method on class Steps,
         and creates a partial for function Steps.
         """
-        if not step:
-            return
-
-        if isinstance(step, Step):
+        if step:
             step.bind(self)
-            return step
-
-        @functools.wraps(step)
-        def bound(path: Path, output_paths: list[Path]):
-            return step(self, path, output_paths)
-        return bound
 
     def find_inputs(self, path: Path):
         """
@@ -118,17 +109,17 @@ class Context:
         """
         input_paths = input_paths or list(self.find_inputs(self.settings['input_dir']))
         # We want to handle tasks in the order they're defined!
-        tasks: dict[BoundStep, list[tuple[Path, list[Path]]]]
-        tasks = {step: [] for rule, step in self.rules if step}
+        tasks: dict[Step, list[tuple[Path, list[Path]]]]
+        tasks = {r.step: [] for r in self.rules if r.step}
 
         for path in _progress(input_paths, 'Planning...'):
-            for rule, step in self.rules:
-                if match := rule.match(path):
+            for rule in self.rules:
+                if match := rule.matcher(self, path):
                     # None can be used to halt further rule processing.
-                    if not step:
+                    if not rule.step:
                         break
                     output_paths: list[Path] = []
-                    for pathcalc in rule.pathcalcs:
+                    for pathcalc in rule.path_calcs:
                         # None can be used to halt further rule processing in
                         # paths as well. This allows a single rule to both do
                         # processing and also halt further processing.
@@ -136,15 +127,15 @@ class Context:
                             break
                         output_paths.append(pathcalc(self, path, match))
                     else:
-                        tasks[step].append((path, output_paths))
+                        tasks[rule.step].append((path, output_paths))
                         # We didn't break above, avoid the break below!
                         continue
-                    tasks[step].append((path, output_paths))
+                    tasks[rule.step].append((path, output_paths))
                     # We need two breaks because we're trying to get out of the
                     # surrounding for loop.
                     break
 
-        flattened: list[tuple[BoundStep, Path, list[Path]]] = []
+        flattened: list[tuple[Step, Path, list[Path]]] = []
         for step, paths in tasks.items():
             flattened.extend((step, p, ops) for p, ops in paths)
 
@@ -172,24 +163,60 @@ class Context:
         self.process(input_paths)
 
 
+class Matcher(t.Generic[T], abc.ABC):
+    """
+    Abstract base class for Path Matchers. Provides pre-baked ability to
+    combine Matchers with | and &.
+    """
+    @abc.abstractmethod
+    def __call__(self, context: Context, path: Path) -> T:
+        ...
+
+    def __or__(self, other: Matcher[T2]):
+        return _OrMatcher(self, other)
+
+    def __and__(self, other: Matcher[T2]):
+        return _AndMatcher(self, other)
+
+
+class _OrMatcher(Matcher[T | T2]):
+    def __init__(self, left: Matcher[T], right: Matcher[T2]):
+        self.left = left
+        self.right = right
+
+    def __call__(self, context: Context, path: Path):
+        return self.left(context, path) or self.right(context, path)
+
+
+class _AndMatcher(Matcher[T | T2]):
+    def __init__(self, left: Matcher[T], right: Matcher[T2]):
+        self.left = left
+        self.right = right
+
+    def __call__(self, context: Context, path: Path):
+        return self.left(context, path) and self.right(context, path)
+
+
+class PathCalc(t.Generic[T], abc.ABC):
+    @abc.abstractmethod
+    def __call__(self, context: Context, path: Path, match: T) -> Path:
+        ...
+
+
 class Rule(t.Generic[T]):
     """
     A single rule for Anchovy file processing, with a matcher, output path
     calculators, and an optional Step to run.
     """
-    match: t.Callable[[Path], T | None]
-    pathcalcs: list[PathCalc[T] ]
-    step: UnboundStep
-
     def __init__(self,
-                 match: t.Callable[[Path], T | None],
-                 pathcalc: t.Sequence[PathCalc[T] | Path ] | PathCalc[T] | Path,
-                 step: UnboundStep = None):
-        self.match = match
+                 matcher: Matcher[T],
+                 path_calc: t.Sequence[PathCalc[T] | Path | None] | PathCalc[T] | Path | None,
+                 step: Step | None = None):
+        self.matcher = matcher
         self.step = step
-        if not isinstance(pathcalc, t.Sequence):
-            pathcalc = [pathcalc]
-        self.pathcalcs = [self._path_to_pathcalc(p) if isinstance(p, Path) else p for p in pathcalc]
+        if not isinstance(path_calc, t.Sequence):
+            path_calc = [path_calc]
+        self.path_calcs = [self._path_to_pathcalc(p) if isinstance(p, Path) else p for p in path_calc]
 
     def _path_to_pathcalc(self, path: Path):
         # pylint: disable=unused-argument
